# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ryu_manager.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11ryu_manager.proto\"\x0c\n\nStopRyuMsg\"A\n\x0bStartRyuMsg\x12\x0c\n\x04port\x18\x01 \x01(\x05\x12\x10\n\x08web_port\x18\x02 \x01(\x05\x12\x12\n\ncontroller\x18\x03 \x01(\t\"\x13\n\x11StopRyuMonitorMsg\"Q\n\x12StartRyuMonitorMsg\x12\x10\n\x08kafka_ip\x18\x01 \x01(\t\x12\x12\n\nkafka_port\x18\x02 \x01(\x05\x12\x15\n\rtime_step_len\x18\x03 \x01(\x05\"\x11\n\x0fGetRyuStatusMsg\"\xa7\x01\n\x06RyuDTO\x12\x13\n\x0bryu_running\x18\x01 \x01(\x08\x12\x17\n\x0fmonitor_running\x18\x02 \x01(\x08\x12\x0c\n\x04port\x18\x03 \x01(\x05\x12\x10\n\x08web_port\x18\x04 \x01(\x05\x12\x12\n\ncontroller\x18\x05 \x01(\t\x12\x10\n\x08kafka_ip\x18\x06 \x01(\t\x12\x12\n\nkafka_port\x18\x07 \x01(\x05\x12\x15\n\rtime_step_len\x18\x08 \x01(\x05\x32\xe5\x01\n\nRyuManager\x12+\n\x0cgetRyuStatus\x12\x10.GetRyuStatusMsg\x1a\x07.RyuDTO\"\x00\x12!\n\x07stopRyu\x12\x0b.StopRyuMsg\x1a\x07.RyuDTO\"\x00\x12#\n\x08startRyu\x12\x0c.StartRyuMsg\x1a\x07.RyuDTO\"\x00\x12/\n\x0estopRyuMonitor\x12\x12.StopRyuMonitorMsg\x1a\x07.RyuDTO\"\x00\x12\x31\n\x0fstartRyuMonitor\x12\x13.StartRyuMonitorMsg\x1a\x07.RyuDTO\"\x00\x62\x06proto3')



_STOPRYUMSG = DESCRIPTOR.message_types_by_name['StopRyuMsg']
_STARTRYUMSG = DESCRIPTOR.message_types_by_name['StartRyuMsg']
_STOPRYUMONITORMSG = DESCRIPTOR.message_types_by_name['StopRyuMonitorMsg']
_STARTRYUMONITORMSG = DESCRIPTOR.message_types_by_name['StartRyuMonitorMsg']
_GETRYUSTATUSMSG = DESCRIPTOR.message_types_by_name['GetRyuStatusMsg']
_RYUDTO = DESCRIPTOR.message_types_by_name['RyuDTO']
StopRyuMsg = _reflection.GeneratedProtocolMessageType('StopRyuMsg', (_message.Message,), {
  'DESCRIPTOR' : _STOPRYUMSG,
  '__module__' : 'ryu_manager_pb2'
  # @@protoc_insertion_point(class_scope:StopRyuMsg)
  })
_sym_db.RegisterMessage(StopRyuMsg)

StartRyuMsg = _reflection.GeneratedProtocolMessageType('StartRyuMsg', (_message.Message,), {
  'DESCRIPTOR' : _STARTRYUMSG,
  '__module__' : 'ryu_manager_pb2'
  # @@protoc_insertion_point(class_scope:StartRyuMsg)
  })
_sym_db.RegisterMessage(StartRyuMsg)

StopRyuMonitorMsg = _reflection.GeneratedProtocolMessageType('StopRyuMonitorMsg', (_message.Message,), {
  'DESCRIPTOR' : _STOPRYUMONITORMSG,
  '__module__' : 'ryu_manager_pb2'
  # @@protoc_insertion_point(class_scope:StopRyuMonitorMsg)
  })
_sym_db.RegisterMessage(StopRyuMonitorMsg)

StartRyuMonitorMsg = _reflection.GeneratedProtocolMessageType('StartRyuMonitorMsg', (_message.Message,), {
  'DESCRIPTOR' : _STARTRYUMONITORMSG,
  '__module__' : 'ryu_manager_pb2'
  # @@protoc_insertion_point(class_scope:StartRyuMonitorMsg)
  })
_sym_db.RegisterMessage(StartRyuMonitorMsg)

GetRyuStatusMsg = _reflection.GeneratedProtocolMessageType('GetRyuStatusMsg', (_message.Message,), {
  'DESCRIPTOR' : _GETRYUSTATUSMSG,
  '__module__' : 'ryu_manager_pb2'
  # @@protoc_insertion_point(class_scope:GetRyuStatusMsg)
  })
_sym_db.RegisterMessage(GetRyuStatusMsg)

RyuDTO = _reflection.GeneratedProtocolMessageType('RyuDTO', (_message.Message,), {
  'DESCRIPTOR' : _RYUDTO,
  '__module__' : 'ryu_manager_pb2'
  # @@protoc_insertion_point(class_scope:RyuDTO)
  })
_sym_db.RegisterMessage(RyuDTO)

_RYUMANAGER = DESCRIPTOR.services_by_name['RyuManager']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _STOPRYUMSG._serialized_start=21
  _STOPRYUMSG._serialized_end=33
  _STARTRYUMSG._serialized_start=35
  _STARTRYUMSG._serialized_end=100
  _STOPRYUMONITORMSG._serialized_start=102
  _STOPRYUMONITORMSG._serialized_end=121
  _STARTRYUMONITORMSG._serialized_start=123
  _STARTRYUMONITORMSG._serialized_end=204
  _GETRYUSTATUSMSG._serialized_start=206
  _GETRYUSTATUSMSG._serialized_end=223
  _RYUDTO._serialized_start=226
  _RYUDTO._serialized_end=393
  _RYUMANAGER._serialized_start=396
  _RYUMANAGER._serialized_end=625
# @@protoc_insertion_point(module_scope)



from typing import Dict, Any, List
import csle_collector.constants.constants as constants


class OSSECIDSAlert:

    def __init__(self, timestamp: float, groups: List[str] = "", host: str = "", ip: str = "", rule_id: str = "",
                 level: int = 1, descr: str = "",
                 src: str = "", user: str = ""):
        """
        A DTO representing an alert from the OSSEC IDS

        :param timestamp: the timestamp of the alert
        :param groups: the groups of the alert
        :param host: the host of the alert
        :param ip: the ip of the alert
        :param rule_id: the rule id that triggered the alert
        :param level: the level of the alert
        :param descr: the description of the alert
        :param src: the source of the alert
        :param user: the user of the alert
        """
        self.timestamp = timestamp
        self.groups = groups
        self.group_ids = list(map(lambda x: self.get_group_id(x), groups))
        self.host = host
        self.ip = ip
        self.rule_id = rule_id
        self.level = level
        self.descr = descr
        self.src = src
        self.user = user

    def get_group_id(self, group: str) -> int:
        """
        Get the id of a group

        :param group: the group to get the id of
        :return: the id
        """
        if group in constants.OSSEC.OSSEC_IDS_ALERT_GROUP_ID:
            return constants.OSSEC.OSSEC_IDS_ALERT_GROUP_ID[group]
        else:
            return 0

    def __str__(self) -> str:
        """
        :return: a string representation of the DTO
        """
        return f"ts: {self.timestamp}, groups: {self.groups}, host: {self.host}, ip: {self.ip}, " \
               f"rule_id: {self.rule_id}, level: {self.level}, descr: {self.descr}, src: {self.src}, " \
               f"user: {self.user}, group_ids: {self.group_ids}"

    @staticmethod
    def from_dict(d: Dict[str, Any]) -> "OSSECIDSAlert":
        """
        Converts a dict representaion of the object into an instance

        :param d: the dict to convert
        :return: the DTO
        """
        obj = OSSECIDSAlert(timestamp=d["timestamp"], groups=d["groups"], host=d["host"], ip=d["ip"],
                            rule_id=d["rule_id"], level=d["level"], descr=d["descr"], src=d["src"], user=d["user"])
        return obj

    def to_dict(self) -> Dict[str, Any]:
        """
        :return: a dict representation of the object
        """
        d = {}
        d["timestamp"] = self.timestamp
        d["groups"] = self.groups
        d["host"] = self.host
        d["ip"] = self.ip
        d["rule_id"] = self.rule_id
        d["level"] = self.level
        d["descr"] = self.descr
        d["src"] = self.src
        d["user"] = self.user
        return d

    def copy(self) -> "OSSECIDSAlert":
        """
        :return: a copy of the DTO
        """
        return OSSECIDSAlert.from_dict(self.to_dict())



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: kafka_manager.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x13kafka_manager.proto\"\x0e\n\x0cStopKafkaMsg\"\x0f\n\rStartKafkaMsg\"\x13\n\x11GetKafkaStatusMsg\"+\n\x08KafkaDTO\x12\x0f\n\x07running\x18\x01 \x01(\x08\x12\x0e\n\x06topics\x18\x02 \x03(\t\"b\n\x0e\x43reateTopicMsg\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x12\n\npartitions\x18\x02 \x01(\x05\x12\x10\n\x08replicas\x18\x03 \x01(\x05\x12\x1c\n\x14retention_time_hours\x18\x04 \x01(\x05\"\x1e\n\x0e\x44\x65leteTopicMsg\x12\x0c\n\x04name\x18\x01 \x01(\t2\xef\x01\n\x0cKafkaManager\x12\x31\n\x0egetKafkaStatus\x12\x12.GetKafkaStatusMsg\x1a\t.KafkaDTO\"\x00\x12\'\n\tstopKafka\x12\r.StopKafkaMsg\x1a\t.KafkaDTO\"\x00\x12)\n\nstartKafka\x12\x0e.StartKafkaMsg\x1a\t.KafkaDTO\"\x00\x12+\n\x0b\x63reateTopic\x12\x0f.CreateTopicMsg\x1a\t.KafkaDTO\"\x00\x12+\n\x0b\x64\x65leteTopic\x12\x0f.DeleteTopicMsg\x1a\t.KafkaDTO\"\x00\x62\x06proto3')



_STOPKAFKAMSG = DESCRIPTOR.message_types_by_name['StopKafkaMsg']
_STARTKAFKAMSG = DESCRIPTOR.message_types_by_name['StartKafkaMsg']
_GETKAFKASTATUSMSG = DESCRIPTOR.message_types_by_name['GetKafkaStatusMsg']
_KAFKADTO = DESCRIPTOR.message_types_by_name['KafkaDTO']
_CREATETOPICMSG = DESCRIPTOR.message_types_by_name['CreateTopicMsg']
_DELETETOPICMSG = DESCRIPTOR.message_types_by_name['DeleteTopicMsg']
StopKafkaMsg = _reflection.GeneratedProtocolMessageType('StopKafkaMsg', (_message.Message,), {
  'DESCRIPTOR' : _STOPKAFKAMSG,
  '__module__' : 'kafka_manager_pb2'
  # @@protoc_insertion_point(class_scope:StopKafkaMsg)
  })
_sym_db.RegisterMessage(StopKafkaMsg)

StartKafkaMsg = _reflection.GeneratedProtocolMessageType('StartKafkaMsg', (_message.Message,), {
  'DESCRIPTOR' : _STARTKAFKAMSG,
  '__module__' : 'kafka_manager_pb2'
  # @@protoc_insertion_point(class_scope:StartKafkaMsg)
  })
_sym_db.RegisterMessage(StartKafkaMsg)

GetKafkaStatusMsg = _reflection.GeneratedProtocolMessageType('GetKafkaStatusMsg', (_message.Message,), {
  'DESCRIPTOR' : _GETKAFKASTATUSMSG,
  '__module__' : 'kafka_manager_pb2'
  # @@protoc_insertion_point(class_scope:GetKafkaStatusMsg)
  })
_sym_db.RegisterMessage(GetKafkaStatusMsg)

KafkaDTO = _reflection.GeneratedProtocolMessageType('KafkaDTO', (_message.Message,), {
  'DESCRIPTOR' : _KAFKADTO,
  '__module__' : 'kafka_manager_pb2'
  # @@protoc_insertion_point(class_scope:KafkaDTO)
  })
_sym_db.RegisterMessage(KafkaDTO)

CreateTopicMsg = _reflection.GeneratedProtocolMessageType('CreateTopicMsg', (_message.Message,), {
  'DESCRIPTOR' : _CREATETOPICMSG,
  '__module__' : 'kafka_manager_pb2'
  # @@protoc_insertion_point(class_scope:CreateTopicMsg)
  })
_sym_db.RegisterMessage(CreateTopicMsg)

DeleteTopicMsg = _reflection.GeneratedProtocolMessageType('DeleteTopicMsg', (_message.Message,), {
  'DESCRIPTOR' : _DELETETOPICMSG,
  '__module__' : 'kafka_manager_pb2'
  # @@protoc_insertion_point(class_scope:DeleteTopicMsg)
  })
_sym_db.RegisterMessage(DeleteTopicMsg)

_KAFKAMANAGER = DESCRIPTOR.services_by_name['KafkaManager']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _STOPKAFKAMSG._serialized_start=23
  _STOPKAFKAMSG._serialized_end=37
  _STARTKAFKAMSG._serialized_start=39
  _STARTKAFKAMSG._serialized_end=54
  _GETKAFKASTATUSMSG._serialized_start=56
  _GETKAFKASTATUSMSG._serialized_end=75
  _KAFKADTO._serialized_start=77
  _KAFKADTO._serialized_end=120
  _CREATETOPICMSG._serialized_start=122
  _CREATETOPICMSG._serialized_end=220
  _DELETETOPICMSG._serialized_start=222
  _DELETETOPICMSG._serialized_end=252
  _KAFKAMANAGER._serialized_start=255
  _KAFKAMANAGER._serialized_end=494
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import csle_collector.elk_manager.elk_manager_pb2 as elk__manager__pb2


class ElkManagerStub(object):
    """Interface exported by the server
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.getElkStatus = channel.unary_unary(
                '/ElkManager/getElkStatus',
                request_serializer=elk__manager__pb2.GetElkStatusMsg.SerializeToString,
                response_deserializer=elk__manager__pb2.ElkDTO.FromString,
                )
        self.stopElk = channel.unary_unary(
                '/ElkManager/stopElk',
                request_serializer=elk__manager__pb2.StopElkMsg.SerializeToString,
                response_deserializer=elk__manager__pb2.ElkDTO.FromString,
                )
        self.startElk = channel.unary_unary(
                '/ElkManager/startElk',
                request_serializer=elk__manager__pb2.StartElkMsg.SerializeToString,
                response_deserializer=elk__manager__pb2.ElkDTO.FromString,
                )
        self.stopElastic = channel.unary_unary(
                '/ElkManager/stopElastic',
                request_serializer=elk__manager__pb2.StopElasticMsg.SerializeToString,
                response_deserializer=elk__manager__pb2.ElkDTO.FromString,
                )
        self.startElastic = channel.unary_unary(
                '/ElkManager/startElastic',
                request_serializer=elk__manager__pb2.StartElasticMsg.SerializeToString,
                response_deserializer=elk__manager__pb2.ElkDTO.FromString,
                )
        self.stopLogstash = channel.unary_unary(
                '/ElkManager/stopLogstash',
                request_serializer=elk__manager__pb2.StopLogstashMsg.SerializeToString,
                response_deserializer=elk__manager__pb2.ElkDTO.FromString,
                )
        self.startLogstash = channel.unary_unary(
                '/ElkManager/startLogstash',
                request_serializer=elk__manager__pb2.StartLogstashMsg.SerializeToString,
                response_deserializer=elk__manager__pb2.ElkDTO.FromString,
                )
        self.stopKibana = channel.unary_unary(
                '/ElkManager/stopKibana',
                request_serializer=elk__manager__pb2.StopKibanaMsg.SerializeToString,
                response_deserializer=elk__manager__pb2.ElkDTO.FromString,
                )
        self.startKibana = channel.unary_unary(
                '/ElkManager/startKibana',
                request_serializer=elk__manager__pb2.StartKibanaMsg.SerializeToString,
                response_deserializer=elk__manager__pb2.ElkDTO.FromString,
                )


class ElkManagerServicer(object):
    """Interface exported by the server
    """

    def getElkStatus(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def stopElk(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def startElk(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def stopElastic(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def startElastic(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def stopLogstash(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def startLogstash(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def stopKibana(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def startKibana(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ElkManagerServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'getElkStatus': grpc.unary_unary_rpc_method_handler(
                    servicer.getElkStatus,
                    request_deserializer=elk__manager__pb2.GetElkStatusMsg.FromString,
                    response_serializer=elk__manager__pb2.ElkDTO.SerializeToString,
            ),
            'stopElk': grpc.unary_unary_rpc_method_handler(
                    servicer.stopElk,
                    request_deserializer=elk__manager__pb2.StopElkMsg.FromString,
                    response_serializer=elk__manager__pb2.ElkDTO.SerializeToString,
            ),
            'startElk': grpc.unary_unary_rpc_method_handler(
                    servicer.startElk,
                    request_deserializer=elk__manager__pb2.StartElkMsg.FromString,
                    response_serializer=elk__manager__pb2.ElkDTO.SerializeToString,
            ),
            'stopElastic': grpc.unary_unary_rpc_method_handler(
                    servicer.stopElastic,
                    request_deserializer=elk__manager__pb2.StopElasticMsg.FromString,
                    response_serializer=elk__manager__pb2.ElkDTO.SerializeToString,
            ),
            'startElastic': grpc.unary_unary_rpc_method_handler(
                    servicer.startElastic,
                    request_deserializer=elk__manager__pb2.StartElasticMsg.FromString,
                    response_serializer=elk__manager__pb2.ElkDTO.SerializeToString,
            ),
            'stopLogstash': grpc.unary_unary_rpc_method_handler(
                    servicer.stopLogstash,
                    request_deserializer=elk__manager__pb2.StopLogstashMsg.FromString,
                    response_serializer=elk__manager__pb2.ElkDTO.SerializeToString,
            ),
            'startLogstash': grpc.unary_unary_rpc_method_handler(
                    servicer.startLogstash,
                    request_deserializer=elk__manager__pb2.StartLogstashMsg.FromString,
                    response_serializer=elk__manager__pb2.ElkDTO.SerializeToString,
            ),
            'stopKibana': grpc.unary_unary_rpc_method_handler(
                    servicer.stopKibana,
                    request_deserializer=elk__manager__pb2.StopKibanaMsg.FromString,
                    response_serializer=elk__manager__pb2.ElkDTO.SerializeToString,
            ),
            'startKibana': grpc.unary_unary_rpc_method_handler(
                    servicer.startKibana,
                    request_deserializer=elk__manager__pb2.StartKibanaMsg.FromString,
                    response_serializer=elk__manager__pb2.ElkDTO.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'ElkManager', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class ElkManager(object):
    """Interface exported by the server
    """

    @staticmethod
    def getElkStatus(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ElkManager/getElkStatus',
            elk__manager__pb2.GetElkStatusMsg.SerializeToString,
            elk__manager__pb2.ElkDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def stopElk(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ElkManager/stopElk',
            elk__manager__pb2.StopElkMsg.SerializeToString,
            elk__manager__pb2.ElkDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def startElk(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ElkManager/startElk',
            elk__manager__pb2.StartElkMsg.SerializeToString,
            elk__manager__pb2.ElkDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def stopElastic(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ElkManager/stopElastic',
            elk__manager__pb2.StopElasticMsg.SerializeToString,
            elk__manager__pb2.ElkDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def startElastic(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ElkManager/startElastic',
            elk__manager__pb2.StartElasticMsg.SerializeToString,
            elk__manager__pb2.ElkDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def stopLogstash(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ElkManager/stopLogstash',
            elk__manager__pb2.StopLogstashMsg.SerializeToString,
            elk__manager__pb2.ElkDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def startLogstash(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ElkManager/startLogstash',
            elk__manager__pb2.StartLogstashMsg.SerializeToString,
            elk__manager__pb2.ElkDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def stopKibana(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ElkManager/stopKibana',
            elk__manager__pb2.StopKibanaMsg.SerializeToString,
            elk__manager__pb2.ElkDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def startKibana(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ElkManager/startKibana',
            elk__manager__pb2.StartKibanaMsg.SerializeToString,
            elk__manager__pb2.ElkDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



import csle_collector.elk_manager.elk_manager_pb2_grpc
import csle_collector.elk_manager.elk_manager_pb2
import csle_collector.constants.constants as constants


def get_elk_status(stub: csle_collector.elk_manager.elk_manager_pb2_grpc.ElkManagerStub,
                   timeout=constants.GRPC.TIMEOUT_SECONDS) \
        -> csle_collector.elk_manager.elk_manager_pb2.ElkDTO:
    """
    Queries the server for the ELK stack status

    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: an ElkDTO describing the status of the ELK stack
    """
    get_elk_status_msg = csle_collector.elk_manager.elk_manager_pb2.GetElkStatusMsg()
    elk_dto = stub.getElkStatus(get_elk_status_msg, timeout=timeout)
    return elk_dto


def stop_elk(stub: csle_collector.elk_manager.elk_manager_pb2_grpc.ElkManagerStub,
             timeout=constants.GRPC.TIMEOUT_SECONDS) \
        -> csle_collector.elk_manager.elk_manager_pb2.ElkDTO:
    """
    Sends a request to the ELK server to stop the whole ELK stack

    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: an ElkDTO describing the status of the ELK stack
    """
    stop_elk_msg = csle_collector.elk_manager.elk_manager_pb2.StopElkMsg()
    elk_dto = stub.stopElk(stop_elk_msg, timeout=timeout)
    return elk_dto


def start_elk(stub: csle_collector.elk_manager.elk_manager_pb2_grpc.ElkManagerStub,
              timeout=constants.GRPC.TIMEOUT_SECONDS) \
        -> csle_collector.elk_manager.elk_manager_pb2.ElkDTO:
    """
    Sends a request to the ELK server to start the ELK stack

    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: an ElkDTO describing the status of the ELK server
    """
    start_elk_msg = csle_collector.elk_manager.elk_manager_pb2.StartElkMsg()
    elk_dto = stub.startElk(start_elk_msg, timeout=timeout)
    return elk_dto


def stop_elastic(stub: csle_collector.elk_manager.elk_manager_pb2_grpc.ElkManagerStub,
                 timeout=constants.GRPC.TIMEOUT_SECONDS) \
        -> csle_collector.elk_manager.elk_manager_pb2.ElkDTO:
    """
    Sends a request to the ELK server to stop Elabticsearch

    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: an ElkDTO describing the status of the ELK stack
    """
    stop_elastic_msg = csle_collector.elk_manager.elk_manager_pb2.StopElasticMsg()
    elk_dto = stub.stopElastic(stop_elastic_msg, timeout=timeout)
    return elk_dto


def start_elastic(stub: csle_collector.elk_manager.elk_manager_pb2_grpc.ElkManagerStub,
                  timeout=constants.GRPC.TIMEOUT_SECONDS) \
        -> csle_collector.elk_manager.elk_manager_pb2.ElkDTO:
    """
    Sends a request to the ELK server to start Elasticsearch

    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: an ElkDTO describing the status of the ELK server
    """
    start_elastic_msg = csle_collector.elk_manager.elk_manager_pb2.StartElkMsg()
    elk_dto = stub.startElastic(start_elastic_msg, timeout=timeout)
    return elk_dto


def stop_kibana(stub: csle_collector.elk_manager.elk_manager_pb2_grpc.ElkManagerStub,
                timeout=constants.GRPC.TIMEOUT_SECONDS) \
        -> csle_collector.elk_manager.elk_manager_pb2.ElkDTO:
    """
    Sends a request to the ELK server to stop Kibana

    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: an ElkDTO describing the status of the ELK stack
    """
    stop_kibana_msg = csle_collector.elk_manager.elk_manager_pb2.StopKibanaMsg()
    elk_dto = stub.stopKibana(stop_kibana_msg, timeout=timeout)
    return elk_dto


def start_kibana(stub: csle_collector.elk_manager.elk_manager_pb2_grpc.ElkManagerStub,
                 timeout=constants.GRPC.TIMEOUT_SECONDS) \
        -> csle_collector.elk_manager.elk_manager_pb2.ElkDTO:
    """
    Sends a request to the ELK server to start Kibana

    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: an ElkDTO describing the status of the ELK server
    """
    start_kibana_msg = csle_collector.elk_manager.elk_manager_pb2.StartKibanaMsg()
    elk_dto = stub.startKibana(start_kibana_msg, timeout=timeout)
    return elk_dto


def stop_logstash(stub: csle_collector.elk_manager.elk_manager_pb2_grpc.ElkManagerStub,
                  timeout=constants.GRPC.TIMEOUT_SECONDS) \
        -> csle_collector.elk_manager.elk_manager_pb2.ElkDTO:
    """
    Sends a request to the ELK server to stop Logstash

    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: an ElkDTO describing the status of the ELK stack
    """
    stop_logstash_msg = csle_collector.elk_manager.elk_manager_pb2.StopKibanaMsg()
    elk_dto = stub.stopLogstash(stop_logstash_msg, timeout=timeout)
    return elk_dto


def start_logstash(stub: csle_collector.elk_manager.elk_manager_pb2_grpc.ElkManagerStub,
                   timeout=constants.GRPC.TIMEOUT_SECONDS) \
        -> csle_collector.elk_manager.elk_manager_pb2.ElkDTO:
    """
    Sends a request to the ELK server to start Logstash

    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: an ElkDTO describing the status of the ELK server
    """
    start_logstash_msg = csle_collector.elk_manager.elk_manager_pb2.StartKibanaMsg()
    elk_dto = stub.startLogstash(start_logstash_msg, timeout=timeout)
    return elk_dto



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: elk_manager.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11\x65lk_manager.proto\"\x12\n\x10StartLogstashMsg\"\x10\n\x0eStartKibanaMsg\"\x11\n\x0fStartElasticMsg\"\x11\n\x0fStopLogstashMsg\"\x0f\n\rStopKibanaMsg\"\x10\n\x0eStopElasticMsg\"\x0c\n\nStopElkMsg\"\r\n\x0bStartElkMsg\"\x11\n\x0fGetElkStatusMsg\"P\n\x06\x45lkDTO\x12\x16\n\x0e\x65lasticRunning\x18\x01 \x01(\x08\x12\x15\n\rkibanaRunning\x18\x02 \x01(\x08\x12\x17\n\x0flogstashRunning\x18\x03 \x01(\x08\x32\x89\x03\n\nElkManager\x12+\n\x0cgetElkStatus\x12\x10.GetElkStatusMsg\x1a\x07.ElkDTO\"\x00\x12!\n\x07stopElk\x12\x0b.StopElkMsg\x1a\x07.ElkDTO\"\x00\x12#\n\x08startElk\x12\x0c.StartElkMsg\x1a\x07.ElkDTO\"\x00\x12)\n\x0bstopElastic\x12\x0f.StopElasticMsg\x1a\x07.ElkDTO\"\x00\x12+\n\x0cstartElastic\x12\x10.StartElasticMsg\x1a\x07.ElkDTO\"\x00\x12+\n\x0cstopLogstash\x12\x10.StopLogstashMsg\x1a\x07.ElkDTO\"\x00\x12-\n\rstartLogstash\x12\x11.StartLogstashMsg\x1a\x07.ElkDTO\"\x00\x12\'\n\nstopKibana\x12\x0e.StopKibanaMsg\x1a\x07.ElkDTO\"\x00\x12)\n\x0bstartKibana\x12\x0f.StartKibanaMsg\x1a\x07.ElkDTO\"\x00\x62\x06proto3')



_STARTLOGSTASHMSG = DESCRIPTOR.message_types_by_name['StartLogstashMsg']
_STARTKIBANAMSG = DESCRIPTOR.message_types_by_name['StartKibanaMsg']
_STARTELASTICMSG = DESCRIPTOR.message_types_by_name['StartElasticMsg']
_STOPLOGSTASHMSG = DESCRIPTOR.message_types_by_name['StopLogstashMsg']
_STOPKIBANAMSG = DESCRIPTOR.message_types_by_name['StopKibanaMsg']
_STOPELASTICMSG = DESCRIPTOR.message_types_by_name['StopElasticMsg']
_STOPELKMSG = DESCRIPTOR.message_types_by_name['StopElkMsg']
_STARTELKMSG = DESCRIPTOR.message_types_by_name['StartElkMsg']
_GETELKSTATUSMSG = DESCRIPTOR.message_types_by_name['GetElkStatusMsg']
_ELKDTO = DESCRIPTOR.message_types_by_name['ElkDTO']
StartLogstashMsg = _reflection.GeneratedProtocolMessageType('StartLogstashMsg', (_message.Message,), {
  'DESCRIPTOR' : _STARTLOGSTASHMSG,
  '__module__' : 'elk_manager_pb2'
  # @@protoc_insertion_point(class_scope:StartLogstashMsg)
  })
_sym_db.RegisterMessage(StartLogstashMsg)

StartKibanaMsg = _reflection.GeneratedProtocolMessageType('StartKibanaMsg', (_message.Message,), {
  'DESCRIPTOR' : _STARTKIBANAMSG,
  '__module__' : 'elk_manager_pb2'
  # @@protoc_insertion_point(class_scope:StartKibanaMsg)
  })
_sym_db.RegisterMessage(StartKibanaMsg)

StartElasticMsg = _reflection.GeneratedProtocolMessageType('StartElasticMsg', (_message.Message,), {
  'DESCRIPTOR' : _STARTELASTICMSG,
  '__module__' : 'elk_manager_pb2'
  # @@protoc_insertion_point(class_scope:StartElasticMsg)
  })
_sym_db.RegisterMessage(StartElasticMsg)

StopLogstashMsg = _reflection.GeneratedProtocolMessageType('StopLogstashMsg', (_message.Message,), {
  'DESCRIPTOR' : _STOPLOGSTASHMSG,
  '__module__' : 'elk_manager_pb2'
  # @@protoc_insertion_point(class_scope:StopLogstashMsg)
  })
_sym_db.RegisterMessage(StopLogstashMsg)

StopKibanaMsg = _reflection.GeneratedProtocolMessageType('StopKibanaMsg', (_message.Message,), {
  'DESCRIPTOR' : _STOPKIBANAMSG,
  '__module__' : 'elk_manager_pb2'
  # @@protoc_insertion_point(class_scope:StopKibanaMsg)
  })
_sym_db.RegisterMessage(StopKibanaMsg)

StopElasticMsg = _reflection.GeneratedProtocolMessageType('StopElasticMsg', (_message.Message,), {
  'DESCRIPTOR' : _STOPELASTICMSG,
  '__module__' : 'elk_manager_pb2'
  # @@protoc_insertion_point(class_scope:StopElasticMsg)
  })
_sym_db.RegisterMessage(StopElasticMsg)

StopElkMsg = _reflection.GeneratedProtocolMessageType('StopElkMsg', (_message.Message,), {
  'DESCRIPTOR' : _STOPELKMSG,
  '__module__' : 'elk_manager_pb2'
  # @@protoc_insertion_point(class_scope:StopElkMsg)
  })
_sym_db.RegisterMessage(StopElkMsg)

StartElkMsg = _reflection.GeneratedProtocolMessageType('StartElkMsg', (_message.Message,), {
  'DESCRIPTOR' : _STARTELKMSG,
  '__module__' : 'elk_manager_pb2'
  # @@protoc_insertion_point(class_scope:StartElkMsg)
  })
_sym_db.RegisterMessage(StartElkMsg)

GetElkStatusMsg = _reflection.GeneratedProtocolMessageType('GetElkStatusMsg', (_message.Message,), {
  'DESCRIPTOR' : _GETELKSTATUSMSG,
  '__module__' : 'elk_manager_pb2'
  # @@protoc_insertion_point(class_scope:GetElkStatusMsg)
  })
_sym_db.RegisterMessage(GetElkStatusMsg)

ElkDTO = _reflection.GeneratedProtocolMessageType('ElkDTO', (_message.Message,), {
  'DESCRIPTOR' : _ELKDTO,
  '__module__' : 'elk_manager_pb2'
  # @@protoc_insertion_point(class_scope:ElkDTO)
  })
_sym_db.RegisterMessage(ElkDTO)

_ELKMANAGER = DESCRIPTOR.services_by_name['ElkManager']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _STARTLOGSTASHMSG._serialized_start=21
  _STARTLOGSTASHMSG._serialized_end=39
  _STARTKIBANAMSG._serialized_start=41
  _STARTKIBANAMSG._serialized_end=57
  _STARTELASTICMSG._serialized_start=59
  _STARTELASTICMSG._serialized_end=76
  _STOPLOGSTASHMSG._serialized_start=78
  _STOPLOGSTASHMSG._serialized_end=95
  _STOPKIBANAMSG._serialized_start=97
  _STOPKIBANAMSG._serialized_end=112
  _STOPELASTICMSG._serialized_start=114
  _STOPELASTICMSG._serialized_end=130
  _STOPELKMSG._serialized_start=132
  _STOPELKMSG._serialized_end=144
  _STARTELKMSG._serialized_start=146
  _STARTELKMSG._serialized_end=159
  _GETELKSTATUSMSG._serialized_start=161
  _GETELKSTATUSMSG._serialized_end=178
  _ELKDTO._serialized_start=180
  _ELKDTO._serialized_end=260
  _ELKMANAGER._serialized_start=263
  _ELKMANAGER._serialized_end=656
# @@protoc_insertion_point(module_scope)



from typing import Dict, Any, Tuple, List
import time


class ClientPopulationMetrics:

    """
    DTO representing information about the client population
    """

    def __init__(self, ip: str = "", ts: float = time.time(), num_clients: int = 0, rate: float = 20,
                 service_time: float = 4):
        """
        Initializes the DTO

        :param ip: the ip address
        :param ts: the timestamp
        :param num_clients: the number of clients currently
        :param rate: the client arrival rate
        :param mean_service_time: the average service time (in terms of time-steps)
        """
        self.ip = ip
        self.ts = ts
        self.num_clients = num_clients
        self.rate = rate
        self.service_time = service_time

    @staticmethod
    def from_kafka_record(record: str) -> "ClientPopulationMetrics":
        """
        Converts a kafka record to a DTO

        :param record: the kafka record
        :return: the DTO
        """
        parts = record.split(",")
        obj = ClientPopulationMetrics(ts=float(parts[0]), ip=parts[1], num_clients=int(parts[2]),
                                      rate=float(parts[3]), service_time=float(parts[4]))
        return obj

    def update_with_kafka_record(self, record: str) -> None:
        """
        Updates the DTO with a new kafka record

        :param record: the kafka record
        :return: None
        """
        parts = record.split(",")
        self.ts = float(parts[0])
        self.ip = parts[1]
        self.num_clients = int(parts[2])
        self.rate = float(parts[3])
        self.service_time = float(parts[4])

    @staticmethod
    def from_dict(d: Dict[str, Any]) -> "ClientPopulationMetrics":
        """
        Converts a dict representation of the object into an instance
        :param d: the dict representation
        :return: the created instance
        """
        rate = 0
        if rate in d:
            rate = d["rate"]
        if "service_time" in d:
            service_time - d["service_time"]
        else:
            service_time = -1
        obj = ClientPopulationMetrics(
            ts=d["ts"], ip=d["ip"], num_clients=d["num_clients"], rate=rate, service_time=service_time)
        return obj

    def to_dict(self) -> Dict[str, Any]:
        """
        :return: a dict representation of the object
        """
        d = {}
        d["ip"] = self.ip
        d["ts"] = self.ts
        d["num_clients"] = self.num_clients
        d["rate"] = self.rate
        d["service_time"] = self.service_time
        return d

    def __str__(self):
        """
        :return: a string representation of the object
        """
        return f"ip: {self.ip}, ts: {self.ts}, num_clients: {self.num_clients}, rate: {self.rate}, " \
               f"avg service time: {self.service_time}"

    def copy(self) -> "ClientPopulationMetrics":
        """
        :return: a copy of the object
        """
        c = ClientPopulationMetrics(ip=self.ip, ts=self.ts, num_clients=self.num_clients, rate=self.rate,
                                    service_time=self.service_time)
        return c

    def get_values(self) -> Tuple[List[float], List[str]]:
        """
        Get the current values

        :return: the values and the labels
        """
        deltas = [int(self.num_clients), float(self.rate), float(self.service_time)]
        labels = ["num_clients", "rate", "service_time"]
        return deltas, labels

    def get_deltas(self, stats_prime: "ClientPopulationMetrics") -> Tuple[List[float], List[str]]:
        """
        Get the deltas between two stats objects

        :param stats_prime: the stats object to compare with
        :return: the deltas and the labels
        """
        deltas = [int(stats_prime.num_clients - self.num_clients), float(stats_prime.rate - self.rate),
                  float(stats_prime.service_time - self.service_time)]
        labels = ["num_clients", "rate", "service_time"]
        return deltas, labels

    def num_attributes(self) -> int:
        """
        :return: The number of attributes of the DTO
        """
        return 5

    @staticmethod
    def schema() -> "ClientPopulationMetrics":
        """
        :return: get the schema of the DTO
        """
        return ClientPopulationMetrics()



from typing import Dict, Any, List
from csle_collector.client_manager.dao.arrival_config import ArrivalConfig
from csle_collector.client_manager.dao.client_arrival_type import ClientArrivalType
import csle_collector.client_manager.client_manager_pb2


class SpikingArrivalConfig(ArrivalConfig):
    """
    DTO representing the configuration of a poisson arrival process with spiking arrivals
    """

    def __init__(self, exponents: List[float], factors: List[float]):
        """
        Initializes the object

        :param exponents: exponents for the spiking arrival rate
        :param factors: factors for the spiking arrival rate
        """
        self.exponents = exponents
        self.factors = factors
        super(SpikingArrivalConfig, self).__init__(client_arrival_type=ClientArrivalType.SPIKING)

    def __str__(self) -> str:
        """
        :return: a string representation of the object
        """
        return f"exponents: {self.exponents}, factors: {self.factors}, client_arrival_type: {self.client_arrival_type}"

    def to_dict(self) -> Dict[str, Any]:
        """
        :return: a dict representation of the object
        """
        d = {}
        d["exponents"] = self.exponents
        d["factors"] = self.factors
        d["client_arrival_type"] = self.client_arrival_type
        return d

    @staticmethod
    def from_dict(d: Dict[str, Any]) -> "SpikingArrivalConfig":
        """
        Converts a dict representation of the object to an instance

        :param d: the dict to convert
        :return: the created instance
        """
        obj = SpikingArrivalConfig(exponents=d["exponents"], factors=d["factors"])
        return obj

    def to_grpc_object(self) -> csle_collector.client_manager.client_manager_pb2.SpikingArrivalConfigDTO:
        """
        :return: a GRPC serializable version of the object
        """
        return csle_collector.client_manager.client_manager_pb2.SpikingArrivalConfigDTO(
            exponents=self.exponents, factors=self.factors
        )

    @staticmethod
    def from_grpc_object(obj: csle_collector.client_manager.client_manager_pb2.SpikingArrivalConfigDTO) \
            -> "SpikingArrivalConfig":
        """
        Instantiates the object from a GRPC DTO

        :param obj: the object to instantiate from
        :return: the instantiated object
        """
        return SpikingArrivalConfig(exponents=obj.exponents, factors=obj.factors)


from typing import List
from math import exp
from math import pow
from math import sin


class EPTMPRateFunction:
    """
    EPTMP or Exponential-Polynomial-Trigonometric rate function having Multiple Periodicities.
    This class is used for creating a rate function that can exhibit both global trends as well as
    periodic components with individual frequencies and amplitudes.
    (Kuhl and Wilson, 1995)
    """

    def __init__(self, thetas: List[float], gammas: List[float], phis: List[float], omegas: List[float]):
        """
        For more information about the EPTMP rate function and its parameters see Section 2.1.1 in the thesis.
        The length of gammas phis and omegas must be the same.

        :param List[float] thetas: Parameters influencing the overall trend over a long timeframe.
        :param List[float] gammas: Amplitudes.
        :param List[float] phis: Shifts.
        :param List[float] omegas: Frequencies.
        """
        self.thetas = thetas
        self.gammas = gammas
        self.phis = phis
        self.omegas = omegas
    
    def rate_f(self, t: float):
        """
        For more information about the EPTMP rate function and its parameters see Section 2.1.1 in the thesis.
        """
        # The first sum in equation 2.1 in the thesis
        theta_sum = 0
        for i, theta in enumerate(self.thetas):
            theta_sum += theta * pow(t, i)
        
        # The second sum in equation 2.1 in the thesis
        second_sum = 0
        for i, (gamma, phi, omega) in enumerate(zip(self.gammas, self.phis, self.omegas)):
            second_sum += gamma * sin(omega * t + phi)

        # Dont't forget to take the exponential
        return exp(theta_sum + second_sum)


import datetime


class SuccessfulLogin:
    """
    DTO Class representing a parsed successful login event from a server in the emulation
    """

    def __init__(self):
        """
        Initializes the DTO
        """
        self.timestamp = None
        self.ip = ""
        self.user = ""

    @staticmethod
    def parse_from_str(login_attempt_str: str, year: int):
        """
        Parses a login event from a string

        :param login_attempt_str: the string to parse
        :param year: the current year
        :return: the parsed login event
        """
        successful_login_attempt_dto = SuccessfulLogin()
        parts = login_attempt_str.split()
        if len(parts) > 6:
            successful_login_attempt_dto.user = parts[0]
            successful_login_attempt_dto.ip = parts[2]
            date_str = " ".join(parts[3:7])
            date_str = str(year) + " " + date_str
            successful_login_attempt_dto.timestamp = \
                datetime.datetime.strptime(date_str, '%Y %a %b %d %H:%M').timestamp()
        return successful_login_attempt_dto

    def __str__(self) -> str:
        """
        :return: a string representation of the object
        """
        return "ip:{}, user:{}, timestamp:{}".format(self.ip, self.user, self.timestamp)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: traffic_manager.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x15traffic_manager.proto\"\x10\n\x0eStopTrafficMsg\"6\n\x0fStartTrafficMsg\x12\x11\n\tsleepTime\x18\x01 \x01(\x05\x12\x10\n\x08\x63ommands\x18\x02 \x03(\t\"\x15\n\x13GetTrafficStatusMsg\"-\n\nTrafficDTO\x12\x0f\n\x07running\x18\x01 \x01(\x08\x12\x0e\n\x06script\x18\x02 \x01(\t2\xa9\x01\n\x0eTrafficManager\x12\x37\n\x10getTrafficStatus\x12\x14.GetTrafficStatusMsg\x1a\x0b.TrafficDTO\"\x00\x12-\n\x0bstopTraffic\x12\x0f.StopTrafficMsg\x1a\x0b.TrafficDTO\"\x00\x12/\n\x0cstartTraffic\x12\x10.StartTrafficMsg\x1a\x0b.TrafficDTO\"\x00\x62\x06proto3')



_STOPTRAFFICMSG = DESCRIPTOR.message_types_by_name['StopTrafficMsg']
_STARTTRAFFICMSG = DESCRIPTOR.message_types_by_name['StartTrafficMsg']
_GETTRAFFICSTATUSMSG = DESCRIPTOR.message_types_by_name['GetTrafficStatusMsg']
_TRAFFICDTO = DESCRIPTOR.message_types_by_name['TrafficDTO']
StopTrafficMsg = _reflection.GeneratedProtocolMessageType('StopTrafficMsg', (_message.Message,), {
  'DESCRIPTOR' : _STOPTRAFFICMSG,
  '__module__' : 'traffic_manager_pb2'
  # @@protoc_insertion_point(class_scope:StopTrafficMsg)
  })
_sym_db.RegisterMessage(StopTrafficMsg)

StartTrafficMsg = _reflection.GeneratedProtocolMessageType('StartTrafficMsg', (_message.Message,), {
  'DESCRIPTOR' : _STARTTRAFFICMSG,
  '__module__' : 'traffic_manager_pb2'
  # @@protoc_insertion_point(class_scope:StartTrafficMsg)
  })
_sym_db.RegisterMessage(StartTrafficMsg)

GetTrafficStatusMsg = _reflection.GeneratedProtocolMessageType('GetTrafficStatusMsg', (_message.Message,), {
  'DESCRIPTOR' : _GETTRAFFICSTATUSMSG,
  '__module__' : 'traffic_manager_pb2'
  # @@protoc_insertion_point(class_scope:GetTrafficStatusMsg)
  })
_sym_db.RegisterMessage(GetTrafficStatusMsg)

TrafficDTO = _reflection.GeneratedProtocolMessageType('TrafficDTO', (_message.Message,), {
  'DESCRIPTOR' : _TRAFFICDTO,
  '__module__' : 'traffic_manager_pb2'
  # @@protoc_insertion_point(class_scope:TrafficDTO)
  })
_sym_db.RegisterMessage(TrafficDTO)

_TRAFFICMANAGER = DESCRIPTOR.services_by_name['TrafficManager']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _STOPTRAFFICMSG._serialized_start=25
  _STOPTRAFFICMSG._serialized_end=41
  _STARTTRAFFICMSG._serialized_start=43
  _STARTTRAFFICMSG._serialized_end=97
  _GETTRAFFICSTATUSMSG._serialized_start=99
  _GETTRAFFICSTATUSMSG._serialized_end=120
  _TRAFFICDTO._serialized_start=122
  _TRAFFICDTO._serialized_end=167
  _TRAFFICMANAGER._serialized_start=170
  _TRAFFICMANAGER._serialized_end=339
# @@protoc_insertion_point(module_scope)



from typing import Dict
from abc import ABC, abstractmethod
from csle_collector.client_manager.dao.client_arrival_type import ClientArrivalType


class ArrivalConfig(ABC):
    """
    Abstract arrival configuration class
    """

    def __init__(self, client_arrival_type: ClientArrivalType) -> None:
        """
        Initializes the object

        :param client_arrival_type: the type of the arrival configuration
        """
        self.client_arrival_type = client_arrival_type

    @abstractmethod
    def to_dict(self) -> Dict:
        """
        :return: a dict representation of the object
        """
        pass

    @staticmethod
    @abstractmethod
    def from_dict(d: Dict) -> "ArrivalConfig":
        """
        Instantiates the object from a dict representation

        :param d: the dict to instantiate the object from
        :return: the instantiated object
        """
        pass

    @abstractmethod
    def to_grpc_object(self):
        """
        :return: a grpc serializable version of the object
        """
        pass

    @staticmethod
    @abstractmethod
    def from_grpc_object(obj):
        """
        Instantiate the object from a GRPC object

        :param obj: the objet to instantiate from
        :return: the instantiated grpc object
        """
        pass

    def to_json_str(self) -> str:
        """
        Converts the DTO into a json string

        :return: the json string representation of the DTO
        """
        import json
        json_str = json.dumps(self.to_dict(), indent=4, sort_keys=True)
        return json_str

    def to_json_file(self, json_file_path: str) -> None:
        """
        Saves the DTO to a json file

        :param json_file_path: the json file path to save  the DTO to
        :return: None
        """
        import io
        json_str = self.to_json_str()
        with io.open(json_file_path, 'w', encoding='utf-8') as f:
            f.write(json_str)

    @staticmethod
    def from_json_file(json_file_path: str) -> "ArrivalConfig":
        """
        Reads a json file and converts it to a DTO

        :param json_file_path: the json file path
        :return: the converted DTO
        """
        import io
        import json
        with io.open(json_file_path, 'r') as f:
            json_str = f.read()
        return ArrivalConfig.from_dict(json.loads(json_str))




from typing import Dict, Any, List
from csle_collector.client_manager.dao.arrival_config import ArrivalConfig
from csle_collector.client_manager.dao.client_arrival_type import ClientArrivalType
import csle_collector.client_manager.client_manager_pb2


class PieceWiseConstantArrivalConfig(ArrivalConfig):
    """
    DTO representing the configuration of a piece-wise constant
    poisson arrival process with exponential service times
    """

    def __init__(self, breakvalues: List[float], breakpoints: List[int]):
        """
        Initializes the object

        :param breakvalues: the constant rates at the different breakpoints
        :param breakpoints: the time steps where the rate changes
        """
        self.breakvalues = breakvalues
        self.breakpoints = breakpoints
        super(PieceWiseConstantArrivalConfig, self).__init__(client_arrival_type=ClientArrivalType.PIECE_WISE_CONSTANT)

    def __str__(self) -> str:
        """
        :return: a string representation of the object
        """
        return f"breakvalues: {self.breakvalues}, breakpoints: {self.breakpoints}, " \
               f"client_arrival_type: {self.client_arrival_type}"

    def to_dict(self) -> Dict[str, Any]:
        """
        :return: a dict representation of the object
        """
        d = {}
        d["breakvalues"] = self.breakvalues
        d["breakpoints"] = self.breakpoints
        d["client_arrival_type"] = self.client_arrival_type
        return d

    @staticmethod
    def from_dict(d: Dict[str, Any]) -> "PieceWiseConstantArrivalConfig":
        """
        Converts a dict representation of the object to an instance

        :param d: the dict to convert
        :return: the created instance
        """
        obj = PieceWiseConstantArrivalConfig(breakvalues=d["breakvalues"], breakpoints=d["breakpoints"])
        return obj

    def to_grpc_object(self) -> csle_collector.client_manager.client_manager_pb2.PieceWiseConstantArrivalConfigDTO:
        """
        :return: a GRPC serializable version of the object
        """
        return csle_collector.client_manager.client_manager_pb2.PieceWiseConstantArrivalConfigDTO(
            breakvalues=self.breakvalues, breakpoints=self.breakpoints
        )

    @staticmethod
    def from_grpc_object(obj: csle_collector.client_manager.client_manager_pb2.PieceWiseConstantArrivalConfigDTO) \
            -> "PieceWiseConstantArrivalConfig":
        """
        Instantiates the object from a GRPC DTO

        :param obj: the object to instantiate from
        :return: the instantiated object
        """
        return PieceWiseConstantArrivalConfig(breakvalues=obj.breakvalues, breakpoints=obj.breakpoints)


from typing import Dict, Any
import csle_collector.elk_manager.elk_manager_pb2


class ElkManagerUtil:
    """
    Class with utility functions related to the ELK Manager
    """

    @staticmethod
    def elk_dto_to_dict(elk_dto: csle_collector.elk_manager.elk_manager_pb2.ElkDTO) -> Dict[str, Any]:
        """
        Converts an ElkDTO to a dict

        :param elk_dto: the dto to convert
        :return: a dict representation of the DTO
        """
        d = {}
        d["kibanaRunning"] = elk_dto.kibanaRunning
        d["elasticRunning"] = elk_dto.elasticRunning
        d["logstashRunning"] = elk_dto.logstashRunning
        return d

    @staticmethod
    def elk_dto_from_dict(d: Dict[str, Any]) -> csle_collector.elk_manager.elk_manager_pb2.ElkDTO:
        """
        Converts a dict representation of an ElkDTO to a DTO

        :param d: the dict to convert
        :return: the converted DTO
        """
        elk_dto = csle_collector.elk_manager.elk_manager_pb2.ElkDTO()
        elk_dto.kibanaRunning = d["kibanaRunning"]
        elk_dto.elasticRunning = d["elasticRunning"]
        elk_dto.logstashRunning = d["logstashRunning"]
        return elk_dto

    @staticmethod
    def elk_dto_empty() -> csle_collector.elk_manager.elk_manager_pb2.ElkDTO:
        """
        :return: an empty ElkDTO
        """
        elk_dto = csle_collector.elk_manager.elk_manager_pb2.ElkDTO()
        elk_dto.kibanaRunning = False
        elk_dto.elasticRunning = False
        elk_dto.logstashRunning = False
        return elk_dto
